/*
 * Copyright (c) 2012 Imre Fazekas.
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *  Neither the name of the Brillien nor the names of its
 *  terms and concepts may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.ignition;

import com.vii.streamline.services.ThreadServices;
import com.vii.streamline.structures.collections.InnerMap;
import com.vii.streamline.web.HttpServices;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

/**
 * This class serves to provide administrative services via terminal
 */
public class TerminalAdmin {

    public static void main(String[] args) {
        if( args.length < 3 ){
            System.out.println("Usage: TerminalAdmin port serverName command" );
        }
        HttpServices.initWebClient();

        try {
            System.out.println("Using commands:" );
            for( int i=0; i<args.length; ++i )
                System.out.println( " <>" + args[i] + "<>" );

            String scheme       = args[0];
            String address      = args[1];
            int port            = Integer.parseInt( args[2] );
            String serverName   = args[3];
            String command      = args[4];

            if( command.equals("stop") ){
                System.out.println("Stopping Brillien server on " + address + " port:" + port + " serverName:" + serverName);
                String stoppingAnswer = HttpServices.sendGetMessage(scheme, address, port, serverName + "/LiaisonControlServlet/stopBrillienInstance", new InnerMap<String, Object>());
                System.out.println("Stopping:" + stoppingAnswer );

                String response;
                do{
                    try{
                        response = HttpServices.sendGetMessage(scheme, "localhost", port, serverName + "/LiaisonControlServlet/pingRunningInstance", new InnerMap<String, Object>());
                    } catch (Exception e){ response = null; }

                    CountDownLatch latch = new CountDownLatch(1);
                    latch.await( 2500, TimeUnit.MILLISECONDS );
                } while( response != null );
            }
        } catch (Exception e) {
            e.printStackTrace(); 
            System.out.println("Usage: TerminalAdmin port serverName command" );
        }

        HttpServices.shutdownWebClient();
        ThreadServices.stop();
    }

}
