/*
 * Copyright (c) 2012 Imre Fazekas.
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *  Neither the name of the Brillien nor the names of its
 *  terms and concepts may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.ignition.transport;

import com.vii.brillien.ignition.transport.amqp.AmqpMediator;
import com.vii.brillien.ignition.transport.jsonrpc.JsonRpcMediator;
import com.vii.brillien.ignition.transport.pure.PureMediator;
import com.vii.brillien.ignition.transport.rest.RestMediator;
import com.vii.brillien.ignition.transport.websocket.WebsocketMediator;
import com.vii.brillien.ignition.transport.xmpp.XmppMediator;
import com.vii.brillien.ignition.transport.xmpp.XmppServices;
import com.vii.streamline.services.error.StreamLineException;
import org.svenson.JSONProperty;

/**
 * Configuration type to define necessary attributes for clients
 */
public class Configuration {

    protected String    senderName;
    protected String    protocol;
    protected String    host;
    protected Integer   port;
    protected String    dispatcherPath;

    /**
     * AMQP-related configuraitons
     */
    protected String    username;
    protected String    password;
    protected String    queuename;
    protected Boolean   durable;

    /**
     * XMPP-related configuraitons
     */
    protected String    domain;
    protected String    resource;

    /**
     * Websocket-related configuraitons
     */
    protected String    wsUrl;
    protected String    keystoreFile;
    protected String    keystorePassword;
    protected String    keystoreType;
    protected String    truststoreFile;
    protected String    truststorePassword;
    protected String    truststoreType;

    {
        protocol = "http";
        host = "localhost";
        durable = true;
        resource = XmppServices.DEFAULT_RESOURCE_NAME;
    }

    @JSONProperty(ignoreIfNull = true)
    public String getSenderName() {
        return senderName;
    }

    public void setSenderName(String senderName) {
        this.senderName = senderName;
    }

    @JSONProperty(ignoreIfNull = true)
    public String getProtocol() {
        return protocol;
    }

    public void setProtocol(String protocol) {
        this.protocol = protocol;
    }

    @JSONProperty(ignoreIfNull = true)
    public String getHost() {
        return host;
    }

    public void setHost(String host) {
        this.host = host;
    }

    @JSONProperty(ignoreIfNull = true)
    public Integer getPort() {
        return port;
    }

    public void setPort(Integer port) {
        this.port = port;
    }

    @JSONProperty(ignoreIfNull = true)
    public String getDispatcherPath() {
        return dispatcherPath;
    }

    public void setDispatcherPath(String dispatcherPath) {
        this.dispatcherPath = dispatcherPath;
    }

    @JSONProperty(ignoreIfNull = true)
    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    @JSONProperty(ignoreIfNull = true)
    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    @JSONProperty(ignoreIfNull = true)
    public String getQueuename() {
        return queuename;
    }

    public void setQueuename(String queuename) {
        this.queuename = queuename;
    }

    @JSONProperty(ignoreIfNull = true)
    public Boolean isDurable() {
        return durable;
    }

    public void setDurable(Boolean durable) {
        this.durable = durable;
    }

    @JSONProperty(ignoreIfNull = true)
    public String getDomain() {
        return domain;
    }

    public void setDomain(String domain) {
        this.domain = domain;
    }

    @JSONProperty(ignoreIfNull = true)
    public String getResource() {
        return resource;
    }

    public void setResource(String resource) {
        this.resource = resource;
    }

    @JSONProperty(ignoreIfNull = true)
    public String getWsUrl() {
        return wsUrl;
    }

    public void setWsUrl(String wsUrl) {
        this.wsUrl = wsUrl;
    }

    @JSONProperty(ignoreIfNull = true)
    public String getKeystoreFile() {
        return keystoreFile;
    }

    public void setKeystoreFile(String keystoreFile) {
        this.keystoreFile = keystoreFile;
    }

    @JSONProperty(ignoreIfNull = true)
    public String getKeystorePassword() {
        return keystorePassword;
    }

    public void setKeystorePassword(String keystorePassword) {
        this.keystorePassword = keystorePassword;
    }

    @JSONProperty(ignoreIfNull = true)
    public String getKeystoreType() {
        return keystoreType;
    }

    public void setKeystoreType(String keystoreType) {
        this.keystoreType = keystoreType;
    }

    @JSONProperty(ignoreIfNull = true)
    public String getTruststoreFile() {
        return truststoreFile;
    }

    public void setTruststoreFile(String truststoreFile) {
        this.truststoreFile = truststoreFile;
    }

    @JSONProperty(ignoreIfNull = true)
    public String getTruststorePassword() {
        return truststorePassword;
    }

    public void setTruststorePassword(String truststorePassword) {
        this.truststorePassword = truststorePassword;
    }

    @JSONProperty(ignoreIfNull = true)
    public String getTruststoreType() {
        return truststoreType;
    }

    public void setTruststoreType(String truststoreType) {
        this.truststoreType = truststoreType;
    }

    public PureMediator createPureMediator() {
        return new PureMediator(senderName, protocol, host, port, dispatcherPath);
    }

    public RestMediator createRestMediator() {
        return new RestMediator(senderName, protocol, host, port, dispatcherPath);
    }

    public JsonRpcMediator createJsonRpcMediator() {
        return new JsonRpcMediator(senderName, protocol, host, port, dispatcherPath);
    }

    public WebsocketMediator createWebsocketMediator() throws StreamLineException {
        return new WebsocketMediator(senderName, wsUrl, keystoreFile, keystorePassword, keystoreType, truststoreFile, truststorePassword, truststoreType);
    }

    public AmqpMediator createAmqpMediator() {
        return new AmqpMediator(host, port, username, password, queuename, durable);
    }

    public XmppMediator createXmppMediator() {
        return new XmppMediator(host, port, domain, username, password, resource);
    }


}
