/*
 * Copyright (c) 2012 Imre Fazekas.
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *  Neither the name of the Brillien nor the names of its
 *  terms and concepts may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.ignition.transport;

import com.vii.brillien.kernel.BrillienException;
import com.vii.streamline.services.ThreadServices;
import com.vii.streamline.web.HttpServices;
import org.apache.http.client.utils.URIBuilder;

/**
 * Abstract specialized Mediator to be specialized by web standard-specific implementations
 */
public abstract class WebMediator extends BrillienMediator<BrillienCommunication> {

    protected String        senderName;
    protected String        protocol;
    protected String        address;
    protected int           port;
    protected String        dispatcherPath;

    protected URIBuilder    builder;


    {
        senderName      = "Tester";
        protocol        = "http";
        address         = "127.0.0.1";
        port            = 7777;
        dispatcherPath  = "/";

        builder         = new URIBuilder();
    }

    public WebMediator(String senderName, String protocol, String address, int port, String dispatcherPath) {
        this.senderName = senderName;
        this.protocol = protocol;
        this.address = address;
        this.port = port;
        this.dispatcherPath = dispatcherPath;

        builder.setHost( address );
        builder.setPort( port );
        builder.setPath( dispatcherPath );
        builder.setScheme( protocol );
    }

    public String getSenderName() {
        return senderName;
    }

    public void setSenderName(String senderName) {
        this.senderName = senderName;
    }

    public String getProtocol() {
        return protocol;
    }

    public void setProtocol(String protocol) {
        this.protocol = protocol;

        builder.setScheme( protocol );
    }

    public String getAddress() {
        return address;
    }

    public void setAddress(String address) {
        this.address = address;

        builder.setHost( address );
    }

    public int getPort() {
        return port;
    }

    public void setPort(int port) {
        this.port = port;

        builder.setPort( port );
    }

    public String getDispatcherPath() {
        return dispatcherPath;
    }

    public void setDispatcherPath(String dispatcherPath) {
        this.dispatcherPath = dispatcherPath;

        builder.setPath( dispatcherPath );
    }

    @Override
    public void connect() throws BrillienException {
        ThreadServices.init();
        HttpServices.initWebClient();
    }

    @Override
    public void disconnect() throws BrillienException {
        HttpServices.shutdownWebClient();
        ThreadServices.stop();
    }

    @Override
    public void startListening() throws BrillienException {
    }

    @Override
    public String mediatorEntity() {
        return senderName;
    }


    @Override
    public String toString() {
        return "WebMediator{" +
                "senderName='" + senderName + '\'' +
                ", protocol='" + protocol + '\'' +
                ", address='" + address + '\'' +
                ", port=" + port +
                ", dispatcherPath='" + dispatcherPath + '\'' +
                '}';
    }
}
