/*
 * Copyright (c) 2012 Imre Fazekas.
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *  Neither the name of the Brillien nor the names of its
 *  terms and concepts may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.ignition.transport.jsonrpc;

import com.vii.brillien.ignition.transport.BrillienCommunication;
import com.vii.brillien.ignition.transport.WebMediator;
import com.vii.brillien.kernel.BrillienException;
import com.vii.streamline.json.JsonRpcResponse;
import com.vii.streamline.services.error.StreamLineException;
import com.vii.streamline.web.HttpServices;

import java.net.URISyntaxException;
import java.util.HashMap;

/**
 * Mediator call handling all JsonRpc-related operations.
 */
public class JsonRpcMediator extends WebMediator {


    {
        dispatcherPath  = "/jsonrpc/BrillienDispatcherServlet";
    }

    public JsonRpcMediator(String senderName, String protocol, String address, int port, String dispatcherPath) {
        super( senderName, protocol, address, port, dispatcherPath );
    }

    @Override
    public BrillienCommunication sendCommunication(long timeout, HashMap<String, Object> properties, BrillienCommunication comm) throws BrillienException {
        try {
            JsonRpcResponse response = HttpServices.sendJsonRpcMessage( builder.build().toString(),
                    comm.getSession().getId(), comm.getApiKey(),
                    comm.getSubject(), comm.getParameters()
            );
            comm.prepareForResponse( response.getError() == null ? response.getResult() : response.getError() );

            return comm;
        } catch (StreamLineException e) {
            throw new BrillienException( e.getErrorCode(), e.getMessage(), e.getPayLoad() );
        } catch (URISyntaxException e) {
            throw new BrillienException( StreamLineException.ERROR_INTERNAL_ERROR, e );
        }
    }

}
